==> py_infraflow_cdk-0.1.9.8.9.7-py3-none-any.whl: infraflow/cdk/lambdas.py <==
import datetime
from typing import Union

from aws_cdk import aws_lambda, Duration
from aws_cdk import aws_sqs
from aws_cdk import aws_lambda_event_sources as sources
from aws_cdk.aws_ec2 import SubnetType, SubnetSelection, Subnet
from aws_cdk.aws_events import Rule, Schedule
from aws_cdk.aws_events_targets import LambdaFunction
from aws_cdk.aws_iam import IRole
from aws_cdk.aws_lambda import IEventSource
from constructs import Construct, IConstruct

from infraflow.cdk.core.service_stage import ServiceStageStack
from infraflow.cdk.core.utils import to_duration
from infraflow.cdk.iam import PolicyBuilder, action_groups
from infraflow.cdk.instrumentation.queues import QueueInstrumentation
from infraflow.cdk.sg.patterns import SecurityGroupTarget
from infraflow.util import caps_camel
from infraflow.cdk.instrumentation.lambdas import LambdaInstrumentation

registry = []


class LambdaContext:
    def __init__(self,
                 stage: ServiceStageStack,
                 path: str,
                 role: IRole = None,
                 runtime: aws_lambda.Runtime = aws_lambda.Runtime.PYTHON_3_9,
                 vpc: bool = True,
                 subnet_type: SubnetType = SubnetType.PRIVATE_WITH_EGRESS,
                 tracing=aws_lambda.Tracing.ACTIVE
                 ):
        self.vpc = vpc
        self.tracing = tracing
        self.subnet_type = subnet_type
        self.role = role
        self.runtime = runtime
        self.path = path
        self.stage = stage
        self.default_sg = stage.default_sg
        self.queues: list[aws_sqs.Queue] = []
        self.functions: list[aws_lambda.Function] = []
        self.queue_instrumentation: dict[aws_sqs.Queue, QueueInstrumentation] = {}
        self.lambda_instrumentation: dict[aws_lambda.Function, LambdaInstrumentation] = {}

    # def to_cdk(self):
    #     return dict(
    #         scope=self.scope,
    #         code=aws_lambda.Code.from_asset(self.path),
    #         runtime=self.runtime
    #     )

    def function(
            self,
            handler: str,
            name=None,
            suffix=None,
            scope_override: IConstruct = None,
            max_concurrency=None,
            timeout: Union[datetime.timedelta, Duration, int]=datetime.timedelta(minutes=5)
    ):
        constructed_name = self.construct_name(handler, name, suffix)
        func = aws_lambda.Function(
                id=constructed_name,
                handler=handler,
                scope=scope_override or self.stage,
                code=aws_lambda.Code.from_asset(self.path),
                runtime=self.runtime,
                reserved_concurrent_executions=max_concurrency,
                timeout=to_duration(timeout),
                security_groups=[
                    self.stage.security_groups.get_group(target=SecurityGroupTarget(
                        scope_override or self.stage,
                        id=constructed_name,
                        cdk_type=aws_lambda.Function,
                        infraflow_pattern=self,
                    ))
                ],
                environment={**self.stage.env.environment_vars},
                vpc_subnets=SubnetSelection(subnets=self.stage.env.service_subnets(self.subnet_type)),
                vpc=self.stage.env.vpc,
                tracing=aws_lambda.Tracing.ACTIVE,
                role=self.role
                # function_name=''
        )
        # if self.role:
        #     func.grant_invoke(self.role)
        self.lambda_instrumentation[func] = LambdaInstrumentation(func, reserved_concurrency=max_concurrency)
        return func

    def construct_name(self, handler: str, name: str, suffix: str):
        base_name = name or caps_camel(handler.split(".")[1])
        constructed_name = caps_camel(f"{base_name}_{suffix}" if suffix else base_name)
        return constructed_name

    def queued_function(
            self,
            handler,
            name=None,
            suffix=None,
            max_concurrency=None,
            batch_size=1,
            timeout: Union[datetime.timedelta, Duration, int]=datetime.timedelta(minutes=5),
            **queue_config
    ):
        constructed_name = self.construct_name(handler, name, suffix)
        queued_function = QueueFunctionConstruct(self.stage, constructed_name)
        queue = aws_sqs.Queue(queued_function, 'Queue', visibility_timeout=to_duration(timeout), **queue_config)
        self.queue_instrumentation[queue] = QueueInstrumentation(queue)
        func = self.function(handler, 'Function', scope_override=queued_function, max_concurrency=max_concurrency, timeout=timeout)
        func.add_event_source(sources.SqsEventSource(queue, batch_size=batch_size))
        queued_function.queue = queue
        queued_function.function = func
        return queued_function

    def scheduled_function(self, handler, schedule: Union[Schedule, Duration, datetime.timedelta], name=None, suffix=None):
        schedule = Duration.seconds(schedule.total_seconds()) if isinstance(schedule, datetime.timedelta) else None
        schedule = Schedule.rate(schedule) if isinstance(schedule, Duration) else None
        constructed_name = self.construct_name(handler, name, suffix)
        func = self.function(handler, name, suffix)
        rule = Rule(self.stage, f"{constructed_name}_schedule", schedule=schedule)
        rule.add_target(LambdaFunction(func))
        return func

    def api_function(self, handler, name=None, suffix=None):
        return self.function(handler, name, suffix)

    def function_with_source(self, handler, event_source: IEventSource, name=None, suffix=None):
        func = self.function(handler, name, suffix)
        func.add_event_source(event_source)
        return func


class QueueFunctionConstruct(Construct):
    function: aws_lambda.Function
    queue: aws_sqs.Queue
    reserved_concurrency: int

    def __init__(self, scope: Construct, id: str):
        super().__init__(scope, id)

    @property
    def function_instrumentation(self):
        return LambdaInstrumentation(self.function, self.reserved_concurrency)

    @property
    def queue_instrumentation(self):
        return QueueInstrumentation(self.queue)

==> py_infraflow_cdk-0.1.9.9.1.4-py3-none-any.whl: infraflow/cdk/lambdas.py <==
import datetime
from typing import Union

from aws_cdk import aws_lambda, Duration
from aws_cdk import aws_sqs
from aws_cdk import aws_lambda_event_sources as sources
from aws_cdk.aws_ec2 import SubnetType, SubnetSelection, Subnet
from aws_cdk.aws_events import Rule, Schedule
from aws_cdk.aws_events_targets import LambdaFunction
from aws_cdk.aws_iam import IRole
from aws_cdk.aws_lambda import IEventSource
from aws_cdk.aws_s3_assets import AssetOptions
from constructs import Construct, IConstruct

from infraflow.cdk.core.service_stage import ServiceStageStack
from infraflow.cdk.core.utils import to_duration
from infraflow.cdk.iam import PolicyBuilder, action_groups
from infraflow.cdk.instrumentation.queues import QueueInstrumentation
from infraflow.cdk.sg.patterns import SecurityGroupTarget
from infraflow.util import caps_camel
from infraflow.cdk.instrumentation.lambdas import LambdaInstrumentation

registry = []


class LambdaContext:
    def __init__(self,
                 stage: ServiceStageStack,
                 path: str,
                 role: IRole = None,
                 runtime: aws_lambda.Runtime = aws_lambda.Runtime.PYTHON_3_9,
                 vpc: bool = True,
                 subnet_type: SubnetType = SubnetType.PRIVATE_WITH_EGRESS,
                 tracing=aws_lambda.Tracing.ACTIVE,
                 excluded_code=None
                 ):
        self.vpc = vpc
        self.tracing = tracing
        self.subnet_type = subnet_type
        self.role = role
        self.runtime = runtime
        self.path = path
        self.stage = stage
        self.default_sg = stage.default_sg
        self.queues: list[aws_sqs.Queue] = []
        self.functions: list[aws_lambda.Function] = []
        self.queue_instrumentation: dict[aws_sqs.Queue, QueueInstrumentation] = {}
        self.lambda_instrumentation: dict[aws_lambda.Function, LambdaInstrumentation] = {}
        self.excluded_code = excluded_code

    # def to_cdk(self):
    #     return dict(
    #         scope=self.scope,
    #         code=aws_lambda.Code.from_asset(self.path),
    #         runtime=self.runtime
    #     )

    def function(
            self,
            handler: str,
            name=None,
            suffix=None,
            scope_override: IConstruct = None,
            max_concurrency=None,
            timeout: Union[datetime.timedelta, Duration, int]=datetime.timedelta(minutes=5)
    ):
        constructed_name = self.construct_name(handler, name, suffix)
        func = aws_lambda.Function(
                id=constructed_name,
                handler=handler,
                scope=scope_override or self.stage,
                code=aws_lambda.Code.from_asset(path=self.path, options=AssetOptions(exclude=self.excluded_code)), ## TODO SH here we can have an exclude pattern
                runtime=self.runtime,
                reserved_concurrent_executions=max_concurrency,
                timeout=to_duration(timeout),
                security_groups=[
                    self.stage.security_groups.get_group(target=SecurityGroupTarget(
                        scope_override or self.stage,
                        id=constructed_name,
                        cdk_type=aws_lambda.Function,
                        infraflow_pattern=self,
                    ))
                ],
                environment={**self.stage.env.environment_vars},
                vpc_subnets=SubnetSelection(subnets=self.stage.env.service_subnets(self.subnet_type)),
                vpc=self.stage.env.vpc,
                tracing=aws_lambda.Tracing.ACTIVE,
                role=self.role
                # function_name=''
        )
        # if self.role:
        #     func.grant_invoke(self.role)
        self.lambda_instrumentation[func] = LambdaInstrumentation(func, reserved_concurrency=max_concurrency)
        return func

    def construct_name(self, handler: str, name: str, suffix: str):
        base_name = name or caps_camel(handler.split(".")[1]) if len(handler.split(".")) > 1 else handler
        constructed_name = caps_camel(f"{base_name}_{suffix}" if suffix else base_name)
        return constructed_name

    def queued_function(
            self,
            handler,
            name=None,
            suffix=None,
            max_concurrency=None,
            batch_size=1,
            timeout: Union[datetime.timedelta, Duration, int]=datetime.timedelta(minutes=5),
            **queue_config
    ):
        constructed_name = self.construct_name(handler, name, suffix)
        queued_function = QueueFunctionConstruct(self.stage, constructed_name)
        queue = aws_sqs.Queue(queued_function, 'Queue', visibility_timeout=to_duration(timeout), **queue_config)
        self.queue_instrumentation[queue] = QueueInstrumentation(queue)
        func = self.function(
            handler,
            'Function',
            scope_override=queued_function,
            max_concurrency=max_concurrency,
            timeout=timeout
        )
        func.add_event_source(sources.SqsEventSource(queue, batch_size=batch_size))
        queued_function.queue = queue
        queued_function.function = func
        return queued_function

    def scheduled_function(self, handler, schedule: Union[Schedule, Duration, datetime.timedelta], name=None, suffix=None):
        schedule = Duration.seconds(schedule.total_seconds()) if isinstance(schedule, datetime.timedelta) else None
        schedule = Schedule.rate(schedule) if isinstance(schedule, Duration) else None
        constructed_name = self.construct_name(handler, name, suffix)
        func = self.function(handler, name, suffix)
        rule = Rule(self.stage, f"{constructed_name}_schedule", schedule=schedule)
        rule.add_target(LambdaFunction(func))
        return func

    def api_function(self, handler, name=None, suffix=None):
        return self.function(handler, name, suffix)

    def function_with_source(self, handler, event_source: IEventSource, name=None, suffix=None):
        func = self.function(handler, name, suffix)
        func.add_event_source(event_source)
        return func


class QueueFunctionConstruct(Construct):
    function: aws_lambda.Function
    queue: aws_sqs.Queue
    reserved_concurrency: int

    def __init__(self, scope: Construct, id: str):
        super().__init__(scope, id)

    @property
    def function_instrumentation(self):
        return LambdaInstrumentation(self.function, self.reserved_concurrency)

    @property
    def queue_instrumentation(self):
        return QueueInstrumentation(self.queue)
